/*
 * 413_adc.c 
 * ADC functions source code 
 *
 * Team: 
 *  Dynamics UPC Manresa
 * 
 * University: 
 *  EPSEM
 * 
 * Author:
 *  David Martn Serrano
 * 
 */

#include "413_adc.h"

/**
 * This functions inits the ADC module of the microcontroller. The programmer
 * must do this function before try an ADC conversion 
 */
void init_ADC(void)
{
    // Program the analog GPIOs
    STEERING_WHEEL_CLUTCH_TRIS  = INPUT;
    STEERING_WHEEL_CLUTCH_ANSEL = ANALOG;
    PLAUSABILITY_4_TRIS         = INPUT;
    PLAUSABILITY_4_ANSEL        = ANALOG;
    PLAUSABILITY_3_TRIS         = INPUT;
    PLAUSABILITY_3_ANSEL        = ANALOG;
    PLAUSABILITY_2_TRIS         = INPUT;
    PLAUSABILITY_2_ANSEL        = ANALOG;
    PLAUSABILITY_1_TRIS         = INPUT;
    PLAUSABILITY_1_ANSEL        = ANALOG;
    SUSPENSION_1_TRIS           = INPUT;
    SUSPENSION_1_ANSEL          = ANALOG;
    SUSPENSION_2_TRIS           = INPUT;
    SUSPENSION_2_ANSEL          = ANALOG;
    SUSPENSION_3_TRIS           = INPUT;
    SUSPENSION_3_ANSEL          = ANALOG;
    SUSPENSION_4_TRIS           = INPUT;
    SUSPENSION_4_ANSEL          = ANALOG;
    OIL_PRESSURE_TRIS           = INPUT;
    OIL_PRESSURE_ANSEL          = ANALOG;
    OIL_TEMPERATURE_TRIS        = INPUT;
    OIL_TEMPERATURE_ANSEL       = ANALOG;
    FUEL_TRIS                   = INPUT;
    FUEL_ANSEL                  = ANALOG;
    HYDRAULIC_TRIS              = INPUT;
    HYDRAULIC_ANSEL             = ANALOG;
    PRESS_TRIS                  = INPUT;
    PRESS_ANSEL                 = ANALOG;
    
    // ADC Lower Threshold LSB and ADC Lower Threshold MSB
    // Set to 0. We won't use the utility of these registers
    ADLTHL = 0x00;
    ADLTHH = 0x00;
    
    // ADC Upper Threshold LSB and ADC Upper Threshold MSB
    // Set to 0. We won't use the utility of these registers
    ADUTHL = 0x00;
    ADUTHH = 0x00;
    
    // ADC Threshold Setpoint Register
    // Set to 0. We won't use the utility of these registers
    ADSTPTL = 0x00;
    ADSTPTH = 0x00;
    
    // ADC Repeat Threshold bits
    // We will only do a unique conversion each time
    ADRPT = 0x00;
    
    // ADC Positive Input Channel Selection bits
    // We will use this register later. Is for select the ADC port
    ADPCH = 0x00;
    
    
    // Additional Sample Capacitor Selection Register
    // We will deactivate the additional capacitance fort the sampling
    ADCAP = 0x00;
    
    // Previous ADC Result Most Significant bits
    // We won't use this feature. Just set to 0
    ADPRE = 0x00;
    
    // ADC Control Register 1
    // Bit 7 --> 0 <-- ADPPOL - Action During 1st Precharge Stage
    //                        - ADPRE=0, so, this bit has no effect
    // Bit 6 --> 0 <-- ADIPEN - A/D Inverted Precharge Enable bit
    //                        - ADDSEN=0, so, this bit has no effect
    // Bit 5 --> 0 <-- ADGPOL - Guard Ring Polarity Selection bit
    //                        - ADC guard Ring outputs start as digital 
    //                          low during Precharge stage
    // Bit 0 --> 0 <-- ADDSEN - Double-Sample Enable bit
    //                        - The result is performed after every conversion
    ADCON1 = 0x00;
      
    // ADC Control Register 2
    // Bit 7 --> 0 <-- ADPSIS - Previous Sample Input Select bits
    //                        - We won't use this capabilities. Set to 0
    // Bit 6,5,4 > 0 < ADCRS  - Accumulated Calculation Right Shift Select bits
    //                        - ADND = 0, so, this bits has no effect
    // Bit 3 --> 0 <-- ADACLR - A/D Accumulator Clear Command bit
    //                        - Bit cleared by hardware. Set to 0. No use
    // Bit 2,1,0 > 0 < ADMD   - ADC Operating Mode Selection bits
    //                        - Basic mode
    ADCON2 = 0x00;
    
    // ADC Control Register 3
    // Bit 6,5,4 > 0 < ADCALC - ADC Error Calculation Mode Select bits
    //                        - Actual CVD result
    // Bit 3 --> 0 <-- ADSOI  - ADC Stop-on-Interrupt bit
    //                        - ADCONT=0, so, this bit has no effect
    // Bit 2,1,0 > 0 < ADTMD  - Threshold Interrupt Mode Select bits
    //                        - Never interrupt
    ADCON3 = 0x00;
    
    // ADC Status Register
    // Read register. No effects.
    ADSTAT = 0x00;
    
    // ADC Reference Selection Register
    // Bit 4 --> 4 <-- ADNREF - ADC Negative Voltage Reference Selection bit
    //                        - Vref- is GND
    // Bit 1,0 -> 0 <- ADPREF - ADC Positive Voltage Reference Selection bits
    //                        - Vref+ is VCC
    ADREF = 0x00;
       
    // ADC AUTO Conversion Trigger Source Selection Register
    // External trigger disabled
    ADACT = 0x00;
    
    // ADC Clock Selection Register
    // Bit 5,4,3,2,1,0 >0x1F< - ADC Conversion Clock Select bits
    //                        - Clock frequency = Fosc/64
    ADCLK = 0x1F;
    
    // ADC Control Register 0
    // Bit 7 --> 1 <-- ADON   - ADC Enable bit
    //                        - Enable the ADC peripheral
    // Bit 6 --> 0 <-- ADCONT - ADC Continuous Operation Enable bit
    //                        - The ADC is cleared at start of each conversion
    // Bit 4 --> 0 <-- ADCS   - ADC Clock Selection bit
    //                        -  Clock supplied by Fosc
    // Bit 2 --> 1 <-- ADFM   - ADC results Format/aligment Selection
    //                        - Sampled data will be right-justified
    // Bit 0 --> 0 <-- ADGO   - ADC Conversion Status bit
    //                        - Don't start a conversion yet
    ADCON0 = 0x84;
    
    // ADC Acquisition Time Control Register
    // Acquisiton time is not included in the data conversion cycle
    ADACQ = 0x00;
}


/**
 * This function returns the value of the selected input.
 * Remember the ADC resolution is 10 bits.
 */
inline uint8_t getADCValue( uint8_t adNumber )
{
    // Select the A/D channel
    ADPCH = adNumber;      
    
    // Delay until capacitors are stabilized
    for( uint8_t i = 0 ; i < 10 ; i++ )    
        asm("nop");
  
    // Start the conversion
    ADCON0bits.ADGO = 1;
    
    // Wait until conversion finishes
    while (ADCON0bits.ADGO);
    
    // Return the result
    return ((uint8_t)((ADRESH << 6) + (ADRESL >> 2)));    
}
