/*
 * 413_clutch_gear.h 
 * Clutch and gear related functions header
 *
 * Team: 
 *  Dynamics UPC Manresa
 * 
 * University: 
 *  EPSEM
 * 
 * Author:
 *  David Martn Serrano
 * 
 */

#ifndef _P413_CLUTCH_GEAR_H
#define _P413_CLUTCH_GEAR_H

#include "413_config.h"
#include "413_pwm.h"

// Define the PWM values for change the gear
#define FAST_GEAR_CHANGE    255
#define SLOW_GEAR_CHANGE    100
#define NO_GEAR_PRESSURE    0

// Define of the GPIOs
#define GEAR_UP_TRIS        TRISHbits.TRISH2
#define GEAR_UP_LAT         LATHbits.LATH2
#define GEAR_DOWN_TRIS      TRISDbits.TRISD0
#define GEAR_DOWN_LAT       LATDbits.LATD0
#define GEAR_DOWN_ANSEL     ANSELDbits.ANSELD0
#define GEAR_0_TRIS         TRISHbits.TRISH1
#define GEAR_0_PORT         PORTHbits.RH1
#define GEAR_0_WPU          WPUHbits.WPUH1
#define GEAR_2_TRIS         TRISAbits.TRISA0
#define GEAR_2_PORT         PORTAbits.RA0
#define GEAR_2_WPU          WPUAbits.WPUA0
#define GEAR_3_TRIS         TRISAbits.TRISA1
#define GEAR_3_PORT         PORTAbits.RA1
#define GEAR_3_WPU          WPUAbits.WPUA1
#define CUTOFF_TRIS         TRISCbits.TRISC6
#define CUTOFF_LAT          LATCbits.LATC6
#define CUTOFF_WPU          WPUCbits.WPUC6

/**
 * This function inits the clutch and gear system. We must do this function
 * before do any work with this system
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void init_clutch_gear(void);

/**
 * Set the power to the pneumatic valve to make air go out from system
 * 
 * Requires:
 *      - A uint8_t variable with a number between 0 and 100 that means the 
 *        % opened of the valve
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void clutchOut(uint8_t percent);

/**
 * Set the power to the pneumatic valve to put air into de system
 * 
 * Requires:
 *      - A uint8_t variable with a number between 0 and 100 that means the 
 *        % opened of the valve
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void clutchIn(uint8_t percent);

/**
 * Activate the normal preassure in the system for fast gear change
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void gearNormalPressure(void);

/**
 * Activate the reduced preassure in the system for slow gear change, and
 * be able to reach the "No gear" position
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void gearNeutralPressure(void);

/**
 * Stops the pressure for change the gear. Is used for stop the current and do
 * not put hot the valve.
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void gearNoPressure(void);

/**
 * Put the gear valve in the position to increment one gear. Be careful with
 * this function because there are not way to know if the movement was finish.
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void gearUp(void);

/**
 * Put the gear valve in the position to decrement one gear. Be careful with
 * this function because there are not way to know if the movement was finish.
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void gearDown(void);

/**
 * Put the gear valve in the passive position. Be careful with
 * this function because there are not way to know if the movement was finish.
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void gearMiddle(void);

/**
 * This function check if the motor is detecting the no gear position
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function return TRUE if there is no gear in the motor. 
 *        FALSE otherwise.
 */
uint8_t checkNoGear(void);

/**
 * This function check if the motor is detecting the second gear
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function return TRUE if there is the second gear in the motor.
 *        FALSE otherwise.
 */
uint8_t checkSecondGear(void);

/**
 * This function check if the motor is detecting the third gear position
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function return TRUE if there is the second gear in the motor. 
 *        FALSE otherwise.
 */
uint8_t checkThirdGear(void);

/**
 * This function deactivte the cutoff. Is a way to increment gear without the
 * clutch
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *     - This function doesn't return anything
 */
void deactivateCuttoff(void);

/**
 * This function deactivte the cutoff. Is a way to increment gear without the
 * clutch
 * 
 * Requires:
 *      - This function no requires anything
 * 
 * Returns:
 *      - This function doesn't return anything
 */
void activateCuttoff(void);

#endif