/*
 * 413_dashboard.c 
 * Dashboard functions source code 
 *
 * Team: 
 *  Dynamics UPC Manresa
 * 
 * University: 
 *  EPSEM
 * 
 * Author:
 *  David Martn Serrano
 * 
 */

#include "413_dashboard.h"


// Variables to save the status of the LEDs
/* Bit 0 - RPM1 and RPM2
 * Bit 1 - RPM3
 * Bit 2 - RPM4
 * Bit 3 - RPM5
 * Bit 4 - RPM9
 * Bit 5 - RPM8
 * Bit 6 - RPM7
 * Bit 7 - RPM6
 */
static volatile uint8_t rpm_value = 0;
/*
 * Bit 0  - A_Out
 * Bit 1  - B_Out
 * Bit 2  - C_Out
 * Bit 3  - D_Out
 * Bit 4  - DOT_Out
 * Bit 5  - G_Out
 * Bit 6  - F_Out
 * Bit 7  - E_Out
 * Bit 8  - Oil temp red LED
 * Bit 9  - Fuel pump relay
 * Bit 10 - Oil pressure red LED
 * Bit 11 - Resv_orange
 * Bit 12 - ETC_APPS
 * Bit 13 - Fuel level red LED
 * Bit 14 - Air pressure red LED
 */
static volatile uint16_t dashboard_value = 0;

/**
 * This functions inits the serial to parallel circuit for handle the
 * dashboard
 */
void init_dashboard(void)
{
    // Configure the ports
    RPM_A_TRIS = OUTPUT;
    RPM_A_ANSEL = DIGITAL;
    RPM_B_TRIS = OUTPUT;
    RPM_B_ANSEL = DIGITAL;
    RPM_CLR_TRIS = OUTPUT;
    RPM_CLR_ANSEL = DIGITAL;
    CLK_TRIS = DIGITAL;
    CLK_ANSEL = DIGITAL;
    DASHBOARD_A_TRIS = OUTPUT;
    DASHBOARD_A_ANSEL = DIGITAL;
    DASHBOARD_B_TRIS = OUTPUT;
    DASHBOARD_B_ANSEL = DIGITAL;
    DASHBOARD_CLR_TRIS = OUTPUT;
    DASHBOARD_CLR_ANSEL = DIGITAL;
    
    // Set the initial values
    RPM_CLR_LAT = 1;
    DASHBOARD_CLR_LAT = 1;
    RPM_B_LAT = 1;
    DASHBOARD_B_LAT = 1;    
    
    // Program the selection of each 7-segments
    SEGMENTS_1_TRIS = OUTPUT;
    SEGMENTS_2_TRIS = OUTPUT;
    SEGMENTS_3_TRIS = OUTPUT;
    SEGMENTS_4_TRIS = OUTPUT;
    SEGMENTS_1_ANSEL = DIGITAL;
    SEGMENTS_2_ANSEL = DIGITAL;
    SEGMENTS_3_ANSEL = DIGITAL;
    SEGMENTS_4_ANSEL = DIGITAL;
    SEGMENTS_1_LAT = 0;
    SEGMENTS_2_LAT = 0;
    SEGMENTS_3_LAT = 0;
    SEGMENTS_4_LAT = 0; 
    
    // Init the preload button ports
    PRELOAD_BUTTON_TRIS = INPUT;
    PRELOAD_BUTTON_ANSEL = DIGITAL;
    PRELOAD_ACTIVATION_TRIS = OUTPUT;
    PRELOAD_ACTIVATION_ANSEL = DIGITAL;
    PRELOAD_ACTIVATION_LAT = 0;
    
    // Update the dashboard with no values (No active LEDs)
    updateDashboard();
}

/**
 * This function updates the values of the dashboard information
 * Uses the local variable "dashboard_value" and the "rpm_value"
 */
void updateDashboard(void)
{
    // Send the saved information to the dashboard
    uint8_t i, temp_rpm;
    uint16_t temp_dashboard;
    
    // Save the local variables
    temp_rpm = rpm_value;
    temp_dashboard = dashboard_value;
    
    // Send the information to the dashboard
    for ( i = 0 ; i < 16 ; i ++)
    {
        // Set to 0 the clock of the serial registers
        CLK_LAT = 0;
        
        // Check if the data we must send is a "0" or "1"
        if ( temp_dashboard & 0x8000 )
            DASHBOARD_A_LAT = 1;
        else
            DASHBOARD_A_LAT = 0;
        
        // Update the rpm value in order
        if ( i > 7 )
        {
            if ( temp_rpm & 0x80 )
                RPM_A_LAT = 1;
            else
                RPM_A_LAT = 0;
        }
        
        // Input the values
        CLK_LAT = 1;
        
        // Rotate temporal variables
        temp_dashboard <<= 1;
        temp_rpm <<= 1;
    }
}

/**
 * This function sets the LED that indicates the state of the fuel pump
 */
void setFuelPump(uint8_t state)
{
    // Activate or deactivate the LED in the internal register
    if ( state )
        dashboard_value |= DASHBOARD_FUEL_PUMP;
    else
        dashboard_value &= ~(DASHBOARD_FUEL_PUMP);
}

/**
 * This function sets the LED that indicates the state of the oil temperature
 */
void setOilTemperature(uint8_t state)
{
    // Activate or deactivate the LED in the internal register
    if ( state )
        dashboard_value |= DASHBOARD_OIL_TEMPERATURE;
    else
        dashboard_value &= ~(DASHBOARD_OIL_TEMPERATURE);
}

/**
 * This function sets the LED that indicates the state of the oil pressure
 */
void setOilPressure(uint8_t state)
{
    // Activate or deactivate the LED in the internal register
    if ( state )
        dashboard_value |= DASHBOARD_OIL_PRESSURE;
    else
        dashboard_value &= ~(DASHBOARD_OIL_PRESSURE);
}

/**
 * This function sets the LED that indicates the state of the Resv system. If
 * this system is not implemented, it will be an extra output to handle systems
 */
void setResvOrange(uint8_t state)
{
    // Activate or deactivate the LED in the internal register
    if ( state )
        dashboard_value |= DASHBOARD_RESV_ORANGE;
    else
        dashboard_value &= ~(DASHBOARD_RESV_ORANGE);
}

/**
 * This function sets the LED that indicates the state of the air pressure
 */
void setAirPressure(uint8_t state)
{
    // Activate or deactivate the LED in the internal register
    if ( state )
        dashboard_value |= DASHBOARD_AIR_PRSSURE;
    else
        dashboard_value &= ~(DASHBOARD_AIR_PRSSURE);
}

/**
 * This function sets the LED that indicates the state of the fuel level
 */
void setFuelLevel(uint8_t state)
{
    // Activate or deactivate the LED in the internal register
    if ( state )
        dashboard_value |= DASHBOARD_FUEL_LEVEL;
    else
        dashboard_value &= ~(DASHBOARD_FUEL_LEVEL);
}

/**
 * This function sets the LED that indicates the state of the security system
 */
void setEtcApps(uint8_t state)
{
    // Activate or deactivate the LED in the internal register
    if ( state )
        dashboard_value |= DASHBOARD_ETC_APPS;
    else
        dashboard_value &= ~(DASHBOARD_ETC_APPS);
}

/**
 * This function sets the LED that indicates the state of the water temperature
 */
void setWaterTemperature(uint8_t state)
{
    // Activate or deactivate the LED in the internal register
    if ( state )
        dashboard_value |= DASHBOARD_WATER_TEMPERATURE;
    else
        dashboard_value &= ~(DASHBOARD_WATER_TEMPERATURE);
}

/**
 * This function sets a number to send to the 7 segments
 */
void setNumberSegments(uint8_t number)
{
    // Limit the maximum value we can send to this function
    if ( number > 9 )
        number = 9;
    
    // Reset the segment number of the 
    dashboard_value &= 0xFF00;
    
    // Depending of the number, set one different case
    switch ( number )
    {
        case 0:
            dashboard_value |= 0xCF;
            break;
        case 1:
            dashboard_value |= 0x06;
            break;
        case 2:
            dashboard_value |= 0xAB; 
            break;    
        case 3:
            dashboard_value |= 0x2F; 
            break;    
        case 4:
            dashboard_value |= 0x66; 
            break;
        case 5:
            dashboard_value |= 0x6D;
            break;
        case 6:
            dashboard_value |= 0xEC;
            break;
        case 7:
            dashboard_value |= 0x07;
            break; 
        case 8:
            dashboard_value |= 0xEF;
            break; 
        case 9:
            dashboard_value |= 0x67;
            break;      
        default:
            break;            
    }
}

/**
 * Sets the rpm value
 */
void setRpm(uint8_t rpm)
{
    // Limit the maximum value we can send to this function
    if ( rpm > 7 )
        rpm = 7;
    
    // Depending of the number, set one different case
    switch ( rpm )
    {
        case 0:
            rpm_value = 0b00000001;
            break;
        case 1:
            rpm_value = 0b00000011;
            break;
        case 2:
            rpm_value = 0b00000111;
            break;    
        case 3:
            rpm_value = 0b00001111;
            break;    
        case 4:
            rpm_value = 0b10001111;
            break;
        case 5:
            rpm_value = 0b11001111;
            break;
        case 6:
            rpm_value = 0b11101111;
            break;
        case 7:
            rpm_value = 0b11111111;
            break;    
        default:
            break;            
    }
}

/**
 * Set the system to activate the gear 7 segments
 */
void setTrisLedOnGear(void)
{
    SEGMENTS_1_LAT = 0;
    SEGMENTS_2_LAT = 0;
    SEGMENTS_3_LAT = 0;
    SEGMENTS_4_LAT = 1;
}

/**
 * Set the system to activate the lowest 7 segments of the motor temperature
 */
void setTrisLedTemperature1(void)
{
    SEGMENTS_1_LAT = 1;
    SEGMENTS_2_LAT = 0;
    SEGMENTS_3_LAT = 0;
    SEGMENTS_4_LAT = 0;
}

/**
 * Set the system to activate the middlest 7 segments of the motor temperature
 */
void setTrisLedTemperature2(void)
{
    SEGMENTS_1_LAT = 0;
    SEGMENTS_2_LAT = 1;
    SEGMENTS_3_LAT = 0;
    SEGMENTS_4_LAT = 0;
}

/**
 * Set the system to activate the highest 7 segments of the motor temperature
 */
void setTrisLedTemperature3(void)
{
    SEGMENTS_1_LAT = 0;
    SEGMENTS_2_LAT = 0;
    SEGMENTS_3_LAT = 1;
    SEGMENTS_4_LAT = 0;
}

/**
 * Returns if the preload button is pressed
 */
uint8_t getPreloadButton(void)
{
    return PORTDbits.RD1;
}

/**
 * Activate the fuel preload
 */
void activatePreload(void)
{
    setFuelPump( TRUE );
    PRELOAD_ACTIVATION_LAT = 1;
}

/**
 * Deactivate the fuel preload
 */
void deactivatePreload(void)
{
    setFuelPump( FALSE );
    PRELOAD_ACTIVATION_LAT = 0;
}