/*
 * 413_mcp.c 
 * MCP2515 related functions source file
 *
 * Team: 
 *  Dynamics UPC Manresa
 * 
 * University: 
 *  EPSEM
 * 
 * Author:
 *  David Martn Serrano
 */

#include "413_mcp.h"


/**
 * This function writes a MCP register using the microcontroller SPI
 */
static
void writeRegister(uint8_t memoryPos, uint8_t data)
{
    enable_CS();
    send_SPI(MCP_WRITE);
    send_SPI(memoryPos);
    send_SPI(data);
    disable_CS();
}

/**
 * This function reads a MCP register using the microcontroller SPI
 */
static
void readRegister(uint8_t memoryPos, uint8_t* data)
{
    enable_CS();
    send_SPI(MCP_READ);
    send_SPI(memoryPos);
    read_SPI(data);
    disable_CS();
}

/**
 * This function resets the MCP
 */
static
void resetMCP(void)
{
    enable_CS();
    send_SPI(MCP_RESET);
    disable_CS();
}


/**
 * This function initialize the CAN system throught the MCP driver
 */
void init_Mcp(uint16_t direction)
{
    uint8_t data;
    uint8_t directionL, directionH;
    
    // Program the inputs for the fast flags
    MCP_RX_FLAG_TRIS = INPUT;
    MCP_TX_FLAG_TRIS = INPUT;
    MCP_RX_FLAG_WPU = TRUE;
    MCP_TX_FLAG_WPU = TRUE;
    
    //Init the SPI module
    init_SPI(); 
    
    // Set the variables for send the direction of the system
    directionL = (uint8_t)(direction & 0b00000111) << 5;
    directionH = (uint8_t)(direction >> 3);
    
    //Configure initial register of MCP
    resetMCP();
    writeRegister(MCP_CNF1,     0b01000100);
    writeRegister(MCP_CNF2,     0b11110010);
    writeRegister(MCP_CNF3,     0b00000100);
    writeRegister(MCP_RXF0SIDH, directionH);
    writeRegister(MCP_RXF0SIDL, directionL);
    writeRegister(MCP_RXF0EID8, 0b00000000);
    writeRegister(MCP_RXF0EID0, 0b00000000);
    writeRegister(MCP_RXM0SIDH, 0b11111111);
    writeRegister(MCP_RXM0SIDL, 0b00000000);
    writeRegister(MCP_RXM0EID8, 0b00000000);
    writeRegister(MCP_RXM0EID0, 0b00000000);
    writeRegister(MCP_RXB0CTRL, 0b00100000);
    writeRegister(MCP_RXB0CTRL, 0b01100000);
    writeRegister(MCP_BFPCTRL,  0b00000000);    
    writeRegister(MCP_TXB0SIDL, 0b00000000);
    writeRegister(MCP_CANINTE,  0b00000101);
    
    // Wait for MCP ready status
    do
    {
        writeRegister(MCP_CANCTRL,  0b00000000);
        readRegister(MCP_CANCTRL,   &data);        
    }while(data != 0b00000000);    
}

/**
 * This functions returns if we can send a new message
 */
inline uint8_t canWeSendCANMessage(void)
{
    return (!MCP_TX_FLAG_PORT);
}

/**
 * This functions check if the MCP received a message
 */
inline uint8_t haveWeReceivedCANMessage(void)
{
    return (!MCP_RX_FLAG_PORT);
}

/**
 * This functions send information to the CAN bus. The maximum bytes to send
 * are 8
 */
uint8_t send_CAN(uint8_t *dataToSend, uint8_t nBytes, uint16_t direction)
{
    // Declare internal variables
    uint8_t data;   
    uint16_t errTimes;
    uint8_t directionL, directionH;
    
    // Cancel the lasts transmissions
    readRegister(MCP_TXB0CTRL, &data);
    writeRegister(MCP_TXB0CTRL, data&~MCP_START_TRANSMIT);
    
    // Clear all the flags
    readRegister(MCP_CANINTF, &data);
    writeRegister(MCP_CANINTF, data & MCP_IS_RECEIVE_BUFFER_FULL);
    readRegister(MCP_CANCTRL, &data);
    writeRegister(MCP_CANCTRL, data | MCP_CANCTRL_ABAT);
    writeRegister(MCP_CANCTRL, data & ~MCP_CANCTRL_ABAT); 
    
    // Clear the sending bit flag
    writeRegister(MCP_TXB0CTRL, 0);
    
    // Load the objective address
    directionL = (uint8_t)(direction & 0b00000111) << 5;
    directionH = (uint8_t)(direction >> 3);
    writeRegister(MCP_TXB0SIDH, directionH);
    writeRegister(MCP_TXB0SIDL, directionL);
    
    // Set the total bytes we will send
    writeRegister(MCP_TXB0DLC, nBytes);
    
    // Set all the data registers
    for(uint8_t i=0; i < nBytes; i++)
    {
        writeRegister(MCP_TXB0D0+i, *(dataToSend++));
    }
    
    // Start the transmissin
    writeRegister(MCP_TXB0CTRL, MCP_START_TRANSMIT);
    
    // Wait until the transmission ends.
    // We will try 4000 times this process
    errTimes = 0;
    do
    {
        readRegister(MCP_TXB0CTRL, &data);
        if( errTimes++ > 4000)
        {
            // Return error
            return ERROR_DETECTED;
        }    
    
    } while((data & MCP_START_TRANSMIT) != MCP_TRANSMIT_DONE);
    
    // Clear the TX flag
    readRegister(MCP_CANINTF, &data);
    writeRegister(MCP_CANINTF, data & 0b11111011);
    
    // Return no error
    return NO_ERROR;
}


/**
 * This functions receive information to the CAN bus. The maximum bytes to send
 * are 8
 */
uint8_t receive_CAN(uint8_t *dataToFill, uint8_t *nBytes)
{
    // Declare internal variables
    uint8_t data;
    uint16_t tryTimes;
    
    // Clear the registers before check if we received a message
    readRegister(MCP_CANINTF, &data);
    writeRegister(MCP_CANINTF, data & MCP_INTERRUPT_FLAGS_RX_TX);
    
    // Check if we received a byte 4000 times
    tryTimes = 0;
    do
    {
        // Read the flag
        readRegister(MCP_CANINTF, &data);
        if(tryTimes++ > 4000)
        {
            return ERROR_DETECTED;
        }
    }while(( data & MCP_IS_RECEIVE_BUFFER_FULL ) == MCP_BUFFER_EMPTY);  
    
    
    // Read the number of bytes we have receive
    readRegister(MCP_RXB0DLC, nBytes);
    
    // Read the data
    for(uint8_t i=0; i < *nBytes; i++)
    {
        readRegister(MCP_RXB0DM+i, dataToFill);
        dataToFill++;
    }
    
    // Clear the flag to receive new interruptions
    writeRegister(MCP_CANINTF, 0); 
    readRegister(MCP_CANINTF, &data);
    writeRegister(MCP_CANINTF, (data & ~MCP_IS_RECEIVE_BUFFER_FULL));    
    
    // Clear the fast flag indicator
    readRegister(MCP_CANINTF, &data);
    writeRegister(MCP_CANINTF, data & 0b11111110);
    
    // Return no error
    return NO_ERROR;
}