/*
 * 413_pwm.c
 * Pwm source code 
 *
 * Team: 
 *  Dynamics UPC Manresa
 * 
 * University: 
 *  EPSEM
 * 
 * Author:
 *  David Martn Serrano
 * 
 */

#include "413_pwm.h"

/**
 * Initialize the timer interrupt
 * This function generate an interrupt each millisecond
 */
void init_pwm(void)
{
    // Programming ports
    CLUTCH_OUT_TRIS = OUTPUT;
    CLUTCH_OUT_ANSEL = DIGITAL;
    CLUTCH_IN_TRIS = OUTPUT;
    CLUTCH_IN_ANSEL = DIGITAL;
    GEAR_OUT_TRIS = OUTPUT;
    GEAR_OUT_ANSEL = DIGITAL;
    
    /*
     * Set TMR2
     */
    // Timer Clock Source Selection Register
    // Clock source is Fosc/4
    T2CLKCON = 0x01;
    
    // Timer Hardware Limit Control Register
    // Bit 7 --> 0 <-- PSYNC  - Timer Prescaler Synchronization Enable bit
    //                        - Timer Prescaler Output is not synchronize FOSC/4
    // Bit 6 --> 0 <-- CPOL   - Timer Clock Polarity Selection bit
    //                        - Rising edge input clock clocks timer/prescaler
    // Bit 5 --> 0 <-- CSYNC  - Timer Clock Synchronization Enable bit
    //                        - ON bit is not synchronized to timer clock input
    // Bit 4,3,2,1,0 >0< MODE - Timer Hardware Limit Control Register
    //                        - Software gate
    T2HLT = 0x00;    
    
    // Timer External Reset Signal Selection Register
    // Bit 3,2,1,0 >0< RSEL   - Timer External Reset Signal Selection Register
    //                        - Not used in ths application
    T2RST = 0x00;
    
    // Timer Period Register
    // Set to 0xF9 for reach 1KHz timer
    T2PR = 0xF9;
    
    // Timer Counter Register
    // When this number reachs T2PR value, timer resets
    T2TMR = 0x00;
    
    // Clearing IF flag
    PIR5bits.TMR2IF = 0;
    
    // Timer2 Control Register
    // Bit 7 --> 1 <-- ON     - Timer On bit
    //                        - Timer is on
    // Bit 6,4,3 >0xE0< CKPS  - Timer Clock Prescale Select bits
    //                        - 1:64 Prescaler
    // Bit 3,2,1,0 >0< OUTPS  - Timer Output Postscaler Select bits
    //                        - 1:1 Postcaler
    T2CON = 0xE0;
    
    
    /*
     * Set PWM6
     */
    // PWM Control Register
    // Bit 7 --> 1 <-- EN     - PWM Module Enable bit
    //                        - PWM mode is enabled
    // Bit 6 --> 0 <-- OUT    - PWM Module Output Level When Bit is Read
    //                        - Set to 0
    // Bit 5 --> 0 <-- POL    - PWM Output Polarity Select bit
    //                        - PWM output is normal
    PWM6CON = 0x80;
    
    // PWM duty value          //0x7CC0 -> 50%
    PWM6DCH = 0x7C;  
    PWM6DCL = 0xC0; 
    
    // CCP Timers Control Register
    // Bit 5,4 -> 0 <- P7TSEL - PWM7 Timer Selection bits
    //                        - PWM7 based on TMR2
    // Bit 3,2 -> 0 <- P6TSEL - PWM6 Timer Selection bits
    //                        - PWM6 based on TMR2
    // Bit 1,0 -> 0 <- CCP5   - CCP5 Timer Selection bits
    //                        - This bit has no effect in this application
    CCPTMRS1 = 0x00;
    
    // Redirect the PWM6 module to the RE6
    RE6PPS = 0x0A;
    
    
    /*
     * Set PWM7
     */
    // PWM Control Register
    // Bit 7 --> 1 <-- EN     - PWM Module Enable bit
    //                        - PWM mode is enabled
    // Bit 6 --> 0 <-- OUT    - PWM Module Output Level When Bit is Read
    //                        - Set to 0
    // Bit 5 --> 0 <-- POL    - PWM Output Polarity Select bit
    //                        - PWM output is normal
    PWM7CON = 0x80;   
    
    // PWM duty value 
    PWM7DCH = 0x7C; 
    PWM7DCL = 0xC0;    
    
    // Redirect the PWM7 module to the RE4
    RE4PPS = 0x0B;
    
    
    /*
     * Set PWM1
     */
    // CCP Control Register
    // Bit 7 --> 1 <-- EN     - CCP Module Enable bit
    //                        - CCP is enabled
    // Bit 5 --> 0 <-- OUT    - CCP Output Data bit (read-only)
    //                        - Read-only bit
    // Bit 4 --> 0 <-- FMT    - CCPW (pulse-width) Value Alignment bit
    //                        - Right-aligned format
    // Bit 3,2,1,0 >0xC< MODE - CCP Mode Select bits
    //                        - PWM Mode
	CCP1CON = 0x8C;    	
    
    // CCP Capture/Compare Time Base
    // Not used in our application
	CCPR1H = 0x00; 
	CCPR1L = 0x00;    
    
	// Select Timer 2 as reference for CCP1
	CCPTMRS0bits.C1TSEL = 0x0;    
    
    // Redirect the CCP1 module to the RE3
    RE3PPS = 0x05;
    CCP1PPS = 0b00100011;
    
}

/**
 * Sets the value for the PMW1
 */ 
void setPwm1(uint16_t value)
{
    value &= 0x03FF;
    
    // Load duty cycle value
    if(CCP1CONbits.FMT)
    {
        value <<= 6;
        CCPR1H = value >> 8;
        CCPR1L = value;
    }
    else
    {
        CCPR1H = value >> 8;
        CCPR1L = value;
    }
}

/**
 * Sets the value for the PMW6
 */ 
void setPwm6(uint16_t value)
{
    // Writing to 8 MSBs of PWM duty cycle in PWMDCH register
     PWM6DCH = (value & 0x03FC)>>2;
     
     // Writing to 2 LSBs of PWM duty cycle in PWMDCL register
     PWM6DCL = (value & 0x0003)<<6;
}

/**
 * Sets the value for the PMW7
 */ 
void setPwm7(uint16_t value)
{
     // Writing to 8 MSBs of PWM duty cycle in PWMDCH register
     PWM7DCH = (value & 0x03FC)>>2;
     
     // Writing to 2 LSBs of PWM duty cycle in PWMDCL register
     PWM7DCL = (value & 0x0003)<<6;
}



