/*
 * 413_spi.c
 * SPI source code 
 *
 * Team: 
 *  Dynamics UPC Manresa
 * 
 * University: 
 *  EPSEM
 * 
 * Author:
 *  David Martn Serrano
 * 
 */

#include "413_spi.h"

/**
 * Initialize SPI peripheral
 */
void init_SPI(void)
{
    // Peripheral Input Selection for SSP1 CLK
    // PORTC -> 0b010 <--
    // PIN 4 -> 0b100 <--
    SSP1CLKPPS = 0b00010100;
    
    // Peripheral Input Selection for SSP1 DAT IN
    // PORTC -> 0b010 <--
    // PIN 3 -> 0b011 <--
    SSP1DATPPS = 0b00010011;
    
    // Peripheral Input Selection for SSP1 DAT OUT
    // PORTC -> 0b010 <--
    // PIN 5 -> 0b101 <--
    SSP1SSPPS = 0b00010101;
    
    // Set RC4 Source selection as SCK
    RC4PPS = 0x19;
            
    // Set RC5 Source selection as SDO
    RC5PPS = 0x1A;
    
    // MSSP Status Register
    // Bit 7 --> 0 <-- SMP    - Slew Rate Control bit
    //                        - Input data is sampled at the middle
    // Bit 6 --> 1 <-- CKE    - Clock select bit
    //                        - Transmit active to Idle clock state
    // Bit 5 --> 0 <-- D/A    - Data/Address bit
    //                        - No used for SPI communication
    // Bit 4 --> 0 <-- P      - Stop Bit
    //                        - No used for SPI communication
    // Bit 3 --> 0 <-- S      - Start bit
    //                        - No used for SPI communication
    // Bit 2 --> 0 <-- R/W    - Read/Write Information
    //                        - No used for SPI communication
    // Bit 1 --> 0 <-- UA     - Update Address bit
    //                        - No used for SPI communication
    // Bit 0 --> 0 <-- BF     - Buffer Full Status bit
    //                        - Don't care, just a read register    
    SSP1STAT = 0x40;
    
    // MSSP Control Register 1
    // Bit 7 --> 0 <-- WCOL   - Write Collision Detect bit
    //                        - Input data is sampled at the middle
    // Bit 6 --> 1 <-- SSPOV  - Receive Overflow Indicator bitt
    //                        - Don't care this bit
    // Bit 5 --> 0 <-- SSPEN  - Master Synchronous Serial Port Enable bit
    //                        - Enables the serial port
    // Bit 4 --> 0 <-- CKP    - SCK Release Control bit
    //                        - Idle state for the clock is a low level
    // Bit 3,2,1,0 >0< SSPM   - Master Synchronous Serial Port Mode Select bits
    //                        - SPI Master mode: Clock = Fosc/64 
    SSP1CON1 = 0x02;
    
    // MSSP Baud Rate Divider and Address Register
    // Divide 16Mhz between 16 to set 1MHz
    SSP1ADD = 16;
    
    // Configure the ports
    CS_TRIS = OUTPUT;
    CS_ANSEL = DIGITAL;
    CS_LAT = FALSE;
    SO_TRIS = OUTPUT;
    SI_TRIS = INPUT;
    SCK_TRIS = OUTPUT;
}



/**
 * SPI send byte function
 * Be careful: is a block function. Returns when the transfer ends
 */
void send_SPI(uint8_t dataToSend)
{
    // Clean the collision bit
    SSP1CON1bits.WCOL = 0;

    // Put the byte to send
    SSP1BUF = dataToSend;

    // Wait until we send the byte
    while(!SSP1STATbits.BF); 
}

/**
 * SPI receive function
 * Be careful: is a block function. Returns when the transfer ends
 */
void read_SPI(uint8_t *dataToFill)
{
    uint8_t dummy;
    
    // Clean the collision bit
    SSP1CON1bits.WCOL = 0;

    // Send a dummy info
    SSP1BUF = dummy;

    // Wait until we receive the byte
    while(!SSP1STATbits.BF);
    
    // Replenish the variable with the received byte
    *dataToFill = SSP1BUF;    
}


/**
  * This function enables the CS of the target system (in this project there
 * is only one extra system
 */
inline void enable_CS()
{
    CS_LAT = TRUE;
}

/**
 * This function disables the CS of the target system (in this project there
 * is only one extra system
 */
inline void disable_CS()
{
    CS_LAT = FALSE;
}
