/*
 * 413_timer_interrupt.c
 * Timer interrupt source code 
 *
 * Team: 
 *  Dynamics UPC Manresa
 * 
 * University: 
 *  EPSEM
 * 
 * Author:
 *  David Martn Serrano
 * 
 */

#include "413_timer_interrupt.h"

/**
 * ETC and TPS VALUES
 */
#define ETC_1_DEVIATION             50
#define ETC_1_FACTOR                1.2
#define ETC_2_DEVIATION             89
#define ETC_2_FACTOR                1.3
#define TPS_1_DEVIATION             12
#define TPS_1_FACTOR                1.7
#define TPS_2_DEVIATION             36
#define TPS_2_FACTOR                1.2
// Clutch defines
#define MINIMUM_CLUTCH_DEVIATION    4
#define CLUTCH_AT_MINIMUM           5
#define CLUTCH_AT_MAXIMUM           95
#define CLUTCH_MOVING_UP_VALUE      80
#define CLUTCH_MOVING_DOWN_VALUE    60
#define CLUTCH_STOP                 0
#define CLUTCH_MAXIMUM              100

/**
 * Initialize the timer interrupt
 * This function generate an interrupt each millisecond
 */
void init_timerInterrupt(void)
{    
    // ADC Control Register 1
    // Bit 7 --> 0 <-- GE     - Timer Gate Enable bit
    //                        - Timer is always counting
    // Bit 6 --> 0 <-- GPOL   - Timer Gate Polarity bit
    //                        - No used in this application
    // Bit 5 --> 0 <-- GTM    - Timer Gate Toggle Mode bit
    //                        - Disable Tiger Gate mode
    // Bit 4 --> 0 <-- GSPM   - Timer Gate Single Pulse Mode bit
    //                        - Time Gate pulse is disabled
    // Bit 3 --> 0 <--GGO/DONE- Timer Gate Single Pulse Acquisition Status bit
    //                        - Not used in this application
    // Bit 2 --> 0 <-- GVAL   - Timer Gate Current State bit
    //                        - Not used in this application
    T1GCON = 0x00;
    
    // Timer Gate Source Selection Register
    // Not used in this application
    T1GATE = 0x00;

    // Timer Clock Source Selection Register
    // Clock source is Fosc
    T1CLK = 0x02;

    // Set the registers to set 1 millisecond
    TMR1H = 0x06;
    TMR1L = 0x00;

    // Clearing IF flag before enabling the interrupt.
    PIR5bits.TMR1IF = 0;

    // Enabling TMR1 interrupt.
    PIE5bits.TMR1IE = 1;

    // ADC Control Register 1
    // Bit 5,4 --> 0 <-- CKPS - Timer Input Clock Prescale Select bits
    //                        - 1:1 Prescale
    // Bit 2 --> 0 <-- SYNC   - Timer External Clock Input Synchro Control bit
    //                        - Synchronize external clock with system clock
    // Bit 1 --> 0 <-- RD16   - 16-Bit Read/Write Mode Enable bit
    //                        - Two 8-bit operations
    // Bit 0 --> 1 <-- ON     - Timer On bit
    //                        - Enables the timer
    T1CON = 0x01;
}

/**
 * Timer 1 ISR
 * This function do all the work
 */
static void _TMR1_ISR()
{
    // Dcleare the local variables
    uint8_t steeringWheelClutch, plausability4,
            plausability3, plausability2, plausability1,
            suspension1, suspension2, suspension3, suspension4,
            oilPressure, oilTemperature, fuel, hydraulic,
            airPressure, etc1, etc2, tps1, tps2;
  
    // Variable for know the difference of the clutch
    uint8_t clutchDifference;    
    
    static volatile uint8_t etc_etc_error_counter = 0;
    static volatile uint8_t etc_tps_error_counter = 0;
    static volatile uint8_t tps_tps_error_counter = 0;
    static volatile uint8_t short_circuit_error_counter = 0; 
    static volatile uint8_t gear_up_flag = FALSE, gear_down_flag = FALSE;
    static volatile uint8_t accel_during_gear_flag = FALSE;
    static volatile uint16_t gear_process_counter = 0, fuel_pump_process = 0;
    static volatile uint8_t gear_down_counter = 0, gear_up_counter = 0;
    static volatile uint8_t fuel_pump_counter = 0, active_segments_counter = 0;
    static volatile uint8_t button1_counter=0;
    static volatile uint8_t wheel_counter = 0;
    static volatile uint8_t wheel1_last = 0, wheel2_last = 0, 
                            wheel3_last = 0, wheel4_last = 0;
    static volatile uint8_t wheel1_counter = 0, wheel2_counter = 0, 
                            wheel3_counter = 0, wheel4_counter = 0;
    static volatile uint16_t show_errors_init = 3000;
    
     // Set the timer again to wait 1 millisecond
    TMR1H = 0x06;
    TMR1L = 0x00;    
    // Clear the TMR1 interrupt flag and start again the count
    PIR5bits.TMR1IF = 0; 
    
    
    // Capture the AD values
    steeringWheelClutch  = getClutchValue();
    plausability4        = getPlaysability4Value();
    plausability3        = getPlaysability3Value();
    plausability2        = getPlaysability2Value();
    plausability1        = getPlaysability1Value();
    suspension1          = getSuspension1Value();
    suspension2          = getSuspension2Value();
    suspension3          = getSuspension3Value();
    suspension4          = getSuspension4Value();
    oilPressure          = getOilPressureValue();
    oilTemperature       = getOilTemperatureValue();
    fuel                 = getFuelLevelValue();
    hydraulic            = getHydraulicPressureValue();
    airPressure          = getAirPressureValue();
    
    // Read the values for the ETC values
    etc1                 = getADCValue( P413_ADC_ETC_SENSOR1 );
    etc2                 = getADCValue( P413_ADC_ETC_SENSOR2 );
    tps1                 = getADCValue( P413_ADC_TPS_SENSOR1 );
    tps2                 = getADCValue( P413_ADC_TPS_SENSOR2 );
    
    // Set the global variables
    global_steeringWheelClutch = ( global_steeringWheelClutch >> 1 ) +
                                 ( steeringWheelClutch >> 1);
    
    global_plausability4 =       ( global_plausability4 >> 1 ) +
                                 ( plausability4 >> 1 );
    global_plausability3 =       ( global_plausability3 >> 1 ) +
                                 ( plausability3 >> 1);
    global_plausability2 =       ( global_plausability2 >> 1 ) +
                                 ( plausability2 >> 1);
    global_plausability1 =       ( global_plausability1 >> 1 ) +
                                 ( plausability1 >> 1 );
    global_suspension1 =         ( global_suspension1 >> 1) +
                                 ( suspension1 >> 1);
    global_suspension2 =         ( global_suspension2 >> 1) +
                                 ( suspension2 >> 1);
    global_suspension3 =         ( global_suspension3 >> 1) +
                                 ( suspension3 >> 1);
    global_suspension4 =         ( global_suspension4 >> 1) +
                                 ( suspension4 >> 1);
    global_oilPressure =         ( global_oilPressure >> 1 ) + 
                                 ( oilPressure >> 1 );
    global_oilTemperature =      ( global_oilTemperature >> 1 ) +
                                 ( oilTemperature >> 1 );
    global_fuel =                ( global_fuel >> 1 ) + ( fuel >> 1 );
    global_hydraulic =           ( global_hydraulic >> 1 ) +
                                 ( hydraulic >> 1);
    global_airPressure =         ( global_airPressure >> 1 ) +
                                 ( airPressure >> 1 );
    global_etc1 =                ( global_etc1 >> 1 ) + ( etc1 >> 1);
    global_etc2 =                ( global_etc2 >> 1 ) + ( etc2 >> 1);
    global_tps1 =                ( global_tps1 >> 1 ) + ( tps1 >> 1 );
    global_tps2 =                ( global_tps2 >> 1 ) + ( tps2 >> 1 );

            
    // Check for errors
    // Check for some value error
    // Oil pressure check
    if ( global_oilPressure > OIL_PRESSURE_ERROR_VALUE)
        setOilPressure( TRUE );
    else
        setOilPressure( FALSE );
    // Oil temperature check
    if ( global_oilTemperature > OIL_TEMPERATURE_ERROR_VALUE)
        setOilTemperature( TRUE );
    else
        setOilTemperature( FALSE );
    // Air pressure check
    if ( global_airPressure <  AIR_PRESSURE_ERROR_VALUE )
        setAirPressure( TRUE );
    else
        setAirPressure( FALSE );
    // Fuel level check
    if ( global_fuel < FUEL_ERROR_VALUE )
        setFuelLevel( TRUE );
    else
        setFuelLevel( FALSE );
    // Water temperature check
    if ( global_water_temperature > WATER_TEMPERATURE_ERROR_VALUE)
        setWaterTemperature( TRUE );
    else
        setWaterTemperature( FALSE );
    
    // If we started the system recently
    if( show_errors_init != 0)
    {
        setFuelLevel( TRUE );
        setAirPressure( TRUE );
        setOilTemperature( TRUE );
        setOilPressure( TRUE );
        show_errors_init--;
    }
    

    // Check the actual gear
    if ( checkNoGear() )
        global_actual_gear = 0;
    else if ( checkSecondGear() )
        global_actual_gear = 2;
    else if ( checkThirdGear() )
        global_actual_gear = 3;
    
    /**
     * ETC Process
     */
    // We will convert all the values in "%"
    etc1 = (etc1 - ETC_1_DEVIATION) * ETC_1_FACTOR;
    etc2 = (etc2 - ETC_2_DEVIATION) * ETC_2_FACTOR; 
    tps1 = (tps1 - TPS_1_DEVIATION) * TPS_1_FACTOR;
    tps2 = (tps2 - TPS_2_DEVIATION) * TPS_2_FACTOR;
    tps2 = 100 - tps2;

    // Check for errors   
    // Check if there is an error between the etc sensors
    if (etc1 > etc2 + 10 || etc2 > etc1 + 10) 
        etc_etc_error_counter++;
    else
        etc_etc_error_counter = 0;
    // Check error between tps sensors
    if (tps1 > tps2 + 10 || tps2 > tps1 + 10) 
        tps_tps_error_counter++;
    else
        tps_tps_error_counter = 0;
    // Check error between tps and etc sensors
    if (tps1 > etc1 + 10 || etc1 > tps1 + 10) 
        etc_tps_error_counter++;
    else
        etc_tps_error_counter = 0;
    // Read the critical signals for check some short circuit
    if ( etc1 < 10 || etc1 > 245 || etc2 < 10 || etc2 > 245 || 
         tps1 < 10 || tps1 > 245 || tps2 < 10 || tps2 > 245 ||
         hydraulic < 10 || hydraulic > 245 || 
         steeringWheelClutch < 10 || steeringWheelClutch > 245)
        short_circuit_error_counter++;
    
    // Check for errors
    if ( etc_etc_error_counter > 100 || tps_tps_error_counter > 100 )
    {
            cutMotorPower();
            setEtcApps( TRUE );
    }
    
    if( etc_tps_error_counter > 200 || short_circuit_error_counter > 100)
    {
        cutPowerSupply();
        setEtcApps( TRUE );
    } 
        
    /**
     * Process for check if the team wants reset the system
     */
    if ( getButton1() )
        button1_counter++;
    else
        button1_counter = 0;
    
    if (button1_counter == 200)
        asm("reset");
    
    
    /***********************************************/
    /****************CHANGE GEAR PROCESS************/
    /***********************************************/
    // If we are changing down the gear do not use the pilot pointer for ETC
    if ( !gear_down_flag || accel_during_gear_flag )
    {
        // Check if we reach or not the position
        if ( tps1 > etc1 )
            etcMotorON();
        else
            etcMotorOFF();
    }
    else
        // If we are doing the gear down process, don't care the normal ETC
        // system
        etcMotorON();
    
    // Check if we are in the process of change the gear
    if ( gear_down_flag || gear_up_flag )
    {
        // Increment the counter for the gear check
        gear_process_counter++;
        // Check if we are in the second step of changing the gear
        if( gear_process_counter <  250)
        {
            if( !global_actual_gear && gear_up_flag)
            {
                activateCuttoff();
            }
            else
            {
                clutchIn( CLUTCH_MAXIMUM );
                clutchOut( CLUTCH_STOP );
            }
        }
        // Check if we are in the third step of changing the gear
        else if ( gear_process_counter < 450 )
        {
            if ( gear_up_flag )
                gearUp();
            else
                gearDown();
        }
        // Check if we are in the fourth step of changing the gear
        else if ( gear_process_counter < 500 )
        {
            accel_during_gear_flag = TRUE;
        }
        // Check if we are in the fifth step of changing the gear
        else if ( gear_process_counter < 650 )
        {
            gearMiddle();
        }
        // Check if we are in the sixth step of changing the gear
        else if ( gear_process_counter < 1000 )
        {
            // Be sure we deactivate the cutoff
            deactivateCuttoff();
            
            // Since now, the system will be accelerated by the pilot
            accel_during_gear_flag = FALSE;
            
            // Mark gear up or gear down
            if( gear_up_flag )
                global_actual_gear++;
            else
                global_actual_gear--;
            
            // Reset flags
            gear_up_flag = FALSE;
            gear_down_flag = FALSE;  
            
            // Save energy stopping the pressure of the gear system
            gearNoPressure();
        }        
    }
    else
    {    
        /***********************************************/
        /*****************CLUTCH PROCESS****************/
        /***********************************************/
        // Calculate the clutch difference betweens the real position and the
        // pilot position
        if ( global_steeringWheelClutch > global_hydraulic )
            clutchDifference = global_steeringWheelClutch - global_hydraulic;
        else
            clutchDifference = global_hydraulic - global_steeringWheelClutch;

        // Check if we must actuate
        if ( clutchDifference > MINIMUM_CLUTCH_DEVIATION )
        {        
            // Actuate the clutch to the correct direction
            if ( global_steeringWheelClutch < CLUTCH_AT_MINIMUM )
            {
                // Put air inside the clutch at maximum value
                clutchIn( CLUTCH_STOP );
                clutchOut( CLUTCH_MAXIMUM );
            }
            else if ( global_steeringWheelClutch > CLUTCH_AT_MAXIMUM )
            {
                // Throw agaw the air inside the clutch at maximum value
                clutchIn( CLUTCH_MAXIMUM );
                clutchOut( CLUTCH_STOP );
            }
            else
            {
                // We will reach this position if the pilot clutch is in a 
                // position outside the minimum or maximum
                if ( global_steeringWheelClutch > global_hydraulic )
                {
                    // Throw agaw the air inside the clutch
                    clutchIn( CLUTCH_MOVING_UP_VALUE );
                    clutchOut( CLUTCH_STOP );
                }
                else
                {
                    // Put air into the clutch
                    clutchIn( CLUTCH_STOP );
                    clutchOut( CLUTCH_MOVING_DOWN_VALUE );
                }
            }
        }
        else
        {
            // We are close to the correct position, stop the work with the
            // pressure in the clutch
            clutchIn( CLUTCH_STOP );
            clutchOut( CLUTCH_STOP );
        }

        /***********************************************/
        /****************GEAR UP PROCESS****************/
        /***********************************************/    
        // Check if we press gear up
        if ( getGearUpButton() )
        {
            if( gear_up_counter++ > 100 )
                gearNormalPressure();                
                if( global_actual_gear == 0 )
                {
                    gear_down_flag = TRUE;
                }
                else                    
                    gear_up_flag = TRUE;
        }            
        else
            gear_up_counter = 0;

        /***********************************************/
        /****************GEAR DOWN PROCESS**************/
        /***********************************************/    
        // Check if we press gear up
        // Block if we have pressed for 100ms the button
        if ( getGearDownButton() && global_actual_gear )
        {
            if( gear_down_counter++ > 100 )
            {
                // If we are going to do a gear down process while we are in the
                // first gear, change for a gear up process with slow gear
                // change
                if ( global_actual_gear == 1 )
                {
                    gear_up_flag = TRUE;
                    gearNeutralPressure();
                }
                else
                {
                    gear_down_flag = TRUE;
                    gearNormalPressure();
                }
            }
        }                    
        // Reset the counter if the button is not pressed
        else
            gear_down_counter = 0;        
    }
    
    // If we press the button for at least 200ms, start 4 seconds preload
    if ( getPreloadButton() )
    {
        if ( fuel_pump_counter++ > 200 )
        {
            activatePreload();
            fuel_pump_process = PRELOAD_TIME_MS;
        }
    }
    // If we reached the 0 value, stop the preload
    if ( !(fuel_pump_process--) )
        deactivatePreload();  
    
    // Process for select the 7 segments to update   
    if ( ++active_segments_counter == 8)
    {
        active_segments_counter = 0;
        if ( ++global_active_segments == 4)
            global_active_segments = 0;
    }
    
    // Wheels process
    // Once each 64 interrupts
    if ( ++wheel_counter == 32 )
    {
        // Check if the number changed
        if( wheel1_last != getWheel1Sensor())
        {
            wheel1_counter ++;
            if ( wheel1_last )
            {
                global_wheel1 = wheel1_counter;
                wheel1_counter = 0;
            }
            wheel1_last ^= 1;
        }
        
         if( wheel2_last != getWheel2Sensor())
        {
            wheel2_counter ++;
            if ( wheel2_last )
            {
                global_wheel2 = wheel2_counter;
                wheel1_counter = 0;
            }
            wheel2_last ^= 1;
        }
        
         if( wheel3_last != getWheel3Sensor())
        {
            wheel3_counter ++;
            if ( wheel3_last )
            {
                global_wheel3 = wheel3_counter;
                wheel3_counter = 0;
            }
            wheel3_last ^= 1;
        }
        
         if( wheel4_last != getWheel4Sensor())
        {
            wheel4_counter ++;
            if ( wheel4_last )
            {
                global_wheel4 = wheel4_counter;
                wheel4_counter = 0;
            }
            wheel4_last ^= 1;
        }
        wheel_counter = 0;
    }
    
}

/**
 * Interrupt vector
 * Once there are an interrupt, the firmware goes to this function
 */
void __interrupt() INTERRUPT_InterruptManager (void)
{
    // Interrupt handler
    if(INTCONbits.PEIE == 1)
    {
        // Check for Timer 1 interrupt
        if(PIE5bits.TMR1IE == 1 && PIR5bits.TMR1IF == 1)
        {
            _TMR1_ISR();
        } 
    }      
    
}




